from selenium.webdriver.common.by import By

from pyallied.web.webWaits import customwebDriverwait


class frameAndWindow(customwebDriverwait):

    def __init__(self, driver):
        # self.driver = driver
        super().__init__(driver)
        self.driver = driver

    def Get_Window_handles(self):
        try:
            return self.driver.window_handles
        except Exception as error:
            raise error

    def Get_window_Position(self):
        try:
            return self.driver.get_window_position()
        except Exception as error:
            raise error

    def Get_window_size(self):
        try:
            return self.driver.get_window_size()
        except Exception as error:
            raise error

    def Minimize_window(self):
        try:
            self.driver.minimize_window()
        except Exception as error:
            raise error

    def Maximize_window(self):
        try:
            self.driver.maximize_window()
        except Exception as error:
            raise error

            # switch to window

    # ex: <a href="somewhere.html" target="windowName">Click here to open a new window</a>

    def switchToWindowUsingName(self, windowName):
        try:
            self.driver.switch_to.window(windowName)
        except Exception as error:
            raise error

    def switchtoWindowUsingHandle(self, windowNumber):
        try:
            current_window = self.driver.current_window_handle
            ListOfHandles = self.driver.window_handles
            if (current_window != ListOfHandles[windowNumber]):
                self.driver.switch_to.window(ListOfHandles[windowNumber])
        except Exception as error:
            raise error
            # frame switch, It’s possible to access subframes by separating the path with a dot,

    # and you can specify the frame by its index too. That is: driver.switch_to_frame("frameName.0.child")
    # would go to the frame named “child” of the first subframe of the frame called “frameName”. All frames are evaluated as if from *top*.

    def switchToFrame(self, xpath):
        try:
            return super().WaitFor_frame_to_be_available_and_switch_to_it(xpath)
        except Exception as error:
            raise error
            # Once we are done with working on frames, we will have to come back to the parent

    # frame which can be done using:

    def switchToParentFrame(self):
        try:
            self.driver.switch_to.default_content()
        except Exception as error:
            raise error

    def switch_To_Frame_ByXpath(self, frameXpath):
        try:
            # self.driver.presence_of_element_located(frameXpath)
            if (self.driver.presence_of_element_located(frameXpath)):
                element = self.driver.find_element(By.XPATH, frameXpath)
                self.driver.switch_to.frame(element)
        except Exception as error:
            raise error

    def switch_To_Frame_ByIndex(self, index: int):
        try:

            self.driver.switch_to.frame(index)
        except Exception as error:
            raise error

    def switch_To_Frame_ByName(self, name):
        try:
            self.driver.switch_to.frame(name)
        except Exception as error:
            raise error

    def switch_To_Parent_Frame(self):
        try:

            self.driver.switch_to.parent_frame()
        except Exception as error:
            raise error
            # navigating



from selenium.webdriver.common.alert import Alert

from pyallied.web.webWaits import customwebDriverwait


class driverAlerts(customwebDriverwait):

    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver

    # alerts
    # Use this class to interact with alert prompts. 
    # It contains methods for dismissing, accepting, inputting, and getting text from alert prompts.   
    def acceptAlert(self):
        try:
            alertPresence = super().WaitForAlert_is_present()
            if (alertPresence):
                Alert(self.driver).accept()
        except Exception as error:
            raise error

    def dismissAlert(self):
        try:
            alertPresence = super().WaitForAlert_is_present()
            if (alertPresence):
                Alert(self.driver).dismiss()
        except Exception as error:
            raise error

    def getAlertText(self):
        try:
            alertPresence = super().WaitForAlert_is_present()
            if (alertPresence):
                alertText = Alert(self.driver).text
                return alertText
        except Exception as error:
            raise error

    def sendKeystoAlert(self, keysToSend):
        try:
            alertPresence = super().WaitForAlert_is_present()
            alert = self.driver.switch_to.alert
            alert.send_keys(keysToSend)
            # name_prompt = Alert(self.driver)
            # name_prompt.send_keys(keysToSend)
        except Exception as error:
            raise error
            # Special Keys

    # The Keys implementation.
    ''' yet to complete'''
    # Touch Actions
    ''' yet to complete'''



from selenium.webdriver.common.by import By
from selenium.webdriver.support.select import Select

from pyallied.web.webWaits import customwebDriverwait


# drop downs


class DropDownActions(customwebDriverwait):
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver

    def selectDropDownByValue(self, xpath, valueToSelect):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                select.select_by_value(valueToSelect)
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def selectDropDownByIndex(self, xpath, indexToSelect):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                select.select_by_index(indexToSelect)
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def selectDropDownByVisibleText(self, xpath, textToSelect):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if (elementPresense):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                select.select_by_visible_text(textToSelect)
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def deselectAllOptionsInDropDown(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                select.deselect_all()
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def getDefaultSelectedDropDownOptions(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            selectedOptions = []
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                all_selected_options = select.all_selected_options
                for i in all_selected_options:
                    if (i.text):
                        selectedOptions.append(i.text)
                    elif (i.get_attribute('value')):
                        selectedOptions.append(i.get_attribute('value'))
                return selectedOptions
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def getAllOptionInDropDown(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            allOptions = []
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                options = select.options
                for i in options:
                    if (i.text):
                        allOptions.append(i.text)
                    elif (i.get_attribute('value')):
                        allOptions.append(i.get_attribute('value'))

                return allOptions
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def deselectByIndex(self, xpath, index):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                select.deselect_by_index(index)
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def deselectByValue(self, xpath, value):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                select.deselect_by_value(value)
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def deselectByVisibleText(self, xpath, text):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                select.deselect_by_visible_text(text)
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def getFirstSelecteOption(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            # elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            firstSelectedOption = []
            if (elementPresense):
                # if(elementVisibility):
                select = Select(self.driver.find_element(By.XPATH, xpath))
                selectedOption = select.first_selected_option
                # for i in selectedOption:
                if (selectedOption.text):
                    firstSelectedOption.append(selectedOption.text)
                elif (selectedOption.get_attribute('value')):
                    firstSelectedOption.append(selectedOption.get_attribute('value'))

                return firstSelectedOption
            else:
                self.__raise_element_not_present_exception(xpath)
        except Exception as error:
            raise error

    def __raise_element_not_present_exception(self, locator):
        raise Exception("element not present : " + locator)

    def __raise_element_not_visible_exception(self, locator):
        raise Exception("element not visible : " + locator)



import setuptools

with open("README.md", "rb") as fh:
    long_description = fh.read().decode("utf8").replace('\r\n', '\n')

setuptools.setup(
    name="pyallied",  # This is the name of the package
    version="1.0.21",  # The initial release version
    author="srinivasaraojyothi@gmail.com",  # Full name of the author
    description="pyallied - a wrapper based on selenium python",
    url="https://github.com/srinivasaraojyothi/seleniumpythonactionbot.git",
    long_description=long_description,  # Long description read from the readme file
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),  # List of all python modules to be installed
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],  # Information to filter the project on PyPi website
    python_requires='>=3.9',  # Minimum version requirement of the package
    py_modules=["dragAndDrop", "alerts", "DropDown", "webWaits", "windowAndFrame", "webElement", "pollWait", "session",
                "mobAppium"],  # Name of the python package
    package_dir={'pyallied.web': 'pyallied/web', 'pyallied.mobile': 'pyallied/mobile'},
    # Directory of the source code of the package
    install_requires=[]  # Install other dependencies if any
)



import os
from datetime import date

import PyPDF2
import easyocr
from PIL import Image, ImageChops, ImageDraw
from imap_tools import AND
from imap_tools import MailBox

from pyallied.web.webWaits import customwebDriverwait


class miscellaneous(customwebDriverwait):
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver

    '''
    returns the embedded text in the image.
    It returns a list of detected text, with each text element containing three types of information. 
    Which are: the text, its bounding box vertices, and the confidence level of the text detection
    '''

    def get_embeddedText_from_image(self, path: str):
        try:
            reader = easyocr.Reader(['en'])
            textList = []
            data = reader.readtext(path)
            for i in data:
                textList.append(i[1])
            # print(type(data),"---obj trype---")
            # return reader.readtext(path)
            return textList
        except Exception as error:
            raise error

    def is_images_same_compare_by_pixel(self, sourceFile, destinationFile):
        try:
            diff_img = ImageChops.difference(Image.open(
                sourceFile), Image.open(destinationFile)).convert('RGB')
            if diff_img.getbbox():
                return False
            else:
                return True
        except Exception as error:
            raise error

    def new_gray(self, size, color):
        img = Image.new('L', size)
        dr = ImageDraw.Draw(img)
        dr.rectangle((0, 0) + size, color)
        return img

    def marking_differences_With_Original_images(self, image_1, image_2, opacity=0.85):
        try:
            img1 = Image.open(
                image_1)
            img2 = Image.open(
                image_2)
            diff = ImageChops.difference(img1, img2)
            diff = diff.convert('L')
            # Hack: there is no threshold in PILL,
            # so we add the difference with itself to do
            # a poor man's thresholding of the mask:
            # (the values for equal pixels-  0 - don't add up)
            thresholded_diff = diff
            for repeat in range(3):
                thresholded_diff = ImageChops.add(
                    thresholded_diff, thresholded_diff)
            h, w = size = diff.size
            mask = self.new_gray(size, int(255 * (opacity)))
            shade = self.new_gray(size, 0)
            new = img1.copy()
            new.paste(shade, mask=mask)
            # To have the original image show partially
            # on the final result, simply put "diff" instead of thresholded_diff bellow
            new.paste(img2, mask=thresholded_diff)
            return new
        except Exception as error:
            raise error

    def pdf_content_reader(self, file1):
        try:
            file = open(file1, 'rb')

            pdfRead = PyPDF2.PdfFileReader(file)
            # print(pdfRead.getDocumentInfo().title,' ----title----')
            output = []
            readData = pdfRead.getPage(0)
            # print(readData.extractText(),' ----single----')
            for page in pdfRead.pages:
                output.append(page.extractText().replace("\n", " "))
            file.close()
            return output
        except Exception as error:
            raise error

    def mail_outlook_search(self, host, port: int, userName, password, searchString: str, searchDate: str = None):
        try:
            mailbox = MailBox(host, port)
            mailbox.login(userName, password)
            # responses =mailbox.idle.wait(timeout=60)
            mailData = {}
            path = ''
            fileName = ''
            # file2 = open(r"D:/Users/sjyothi/Repos/pythonseleniumFramework/testdata/MyFile2.txt", "w+")
            # data=''
            # if responses:
            # print(responses,'-----respon')
            # for msg in mailbox.fetch():
            if (searchDate != None):
                spliData = searchDate.split("/")
                year = int(spliData[0])
                month = int(spliData[1])
                day = int(spliData[2])
                for msg in mailbox.fetch(AND(date=date(year, month, day))):
                    if (msg.subject in searchString):
                        # print(msg.attachments,'--attach---')
                        if (msg.attachments):
                            # print(msg.date.strftime,"----------------yes--------")
                            cwd = os.getcwd()
                            if (not os.path.exists(cwd + "/tmp")):
                                path = os.mkdir(cwd + "/tmp")
                                # print(path,"---path---")
                            else:
                                path = cwd + "/tmp"
                            for att in msg.attachments:
                                # print(pd.DataFrame(data=att.payload),"-----------------------------------")

                                with open(path + '/{}'.format(att.filename), 'wb') as f:
                                    # print('----',att.filename)
                                    fileName = att.filename
                                    f.write(att.payload)

                                mailData[msg.date_str] = msg.text.replace("\r\n",
                                                                          " "), msg.subject, self.pdf_content_reader(
                                    path + "/" + fileName)
                        else:
                            # print(msg.date_str.replace(" ",""),"----------------yes--------")

                            mailData[msg.date_str] = msg.text.replace(
                                "\r\n", " "), msg.subject
                        # break
                # self.pdf_content_reader(path+"/"+fileName)
            else:
                searchDate = date.today()
                Year = date.today().year
                month = date.today().month
                day = date.today().day
                for msg in mailbox.fetch(AND(date=date(Year, month, day))):
                    if (msg.subject in searchString):
                        # print(msg.attachments,'--attach---')
                        if (msg.attachments):
                            # print(msg.date.strftime,"----------------yes--------")
                            cwd = os.getcwd()
                            if (not os.path.exists(cwd + "/tmp")):
                                path = os.mkdir(cwd + "/tmp")
                                # print(path,"---path---")
                            else:
                                path = cwd + "/tmp"
                            for att in msg.attachments:
                                # print(pd.DataFrame(data=att.payload),"-----------------------------------")

                                with open(path + '/{}'.format(att.filename), 'wb') as f:
                                    # print('----',att.filename)
                                    fileName = att.filename
                                    f.write(att.payload)

                                mailData[msg.date_str] = msg.text.replace("\r\n",
                                                                          " "), msg.subject, self.pdf_content_reader(
                                    path + "/" + fileName)
                        else:
                            # print(msg.date.time+"_"+msg.date.today,"----------------yes--------")

                            mailData[msg.date_str] = msg.text.replace(
                                "\r\n", " "), msg.subject
            mailbox.logout()
            return mailData

        except Exception as error:
            raise error

    def mail_outlook_Attachments(self, host, port: int, userName, password, searchString: str, fromAddress: str):
        try:
            mailbox = MailBox(host, port)
            mailbox.login(userName, password, 'INBOX')
            responses = mailbox.idle.wait(timeout=120)
            attachment = []
            if responses:
                for msg in mailbox.fetch(AND(text=searchString, new=True, from_=fromAddress)):
                    for attach in msg.attachments:
                        attachment.append(attach.filename)
            return attachment
        except Exception as error:
            raise error

    def mail_outlook_Attachments_Read(self, host, port: int, userName, password, searchString: str, fromAddress: str):
        try:
            mailbox = MailBox(host, port)
            mailbox.login(userName, password, 'INBOX')
            responses = mailbox.idle.wait(timeout=120)
            attachment_Read = {}
            if responses:
                for msg in mailbox.fetch(AND(text=searchString, new=True, from_=fromAddress)):
                    for attach in msg.attachments:
                        attachment_Read[attach.filename] = attach.payload
                        # attachment_Read.append(attach.payload)
            return attachment_Read
        except Exception as error:
            raise error
